indexing

	description: "Class that implements the basics of an HTTP client."

	todos:
		"1. raise exception when Content-Length and number of octects in message body do not match (RFC2616, section 4.4.)."

	author: "Berend de Boer"
	date: "$Date: 2006/06/01 $"
	revision: "$Revision: #1 $"


deferred class

	EPX_HTTP_CLIENT


inherit

	EPX_TCP_CLIENT_BASE
		rename
			make as make_authenticate,
			make_with_port as make_authenticate_with_port,
			service as http_service,
			socket as http
		redefine
			close,
			open
		end

	EPX_REPLY_CODE
		rename
			reply_code as response_code
		export
			{NONE} response_code, is_line_with_reply_code
		end


feature {NONE} -- Initialization

	make (a_server_name: STRING) is
			-- Prepare for request to `a_server_name'.
		require
			valid_server_name: a_server_name /= Void and then not a_server_name.is_empty
		do
			make_with_port (a_server_name, 0)
		end

	make_from_port (a_server_name: STRING; a_port: INTEGER) is
		obsolete "2004-12-16 use make_with_port instead."
		do
			make_with_port (a_server_name, a_port)
		end

	make_with_port (a_server_name: STRING; a_port: INTEGER) is
			-- Prepare for request.
			-- Use `port' is 0 to use the default port (80).
		require
			valid_server_name: a_server_name /= Void and then not a_server_name.is_empty
			valid_port: a_port >= 0 and then a_port <= 65535
		do
			make_authenticate_with_port (a_server_name, a_port, Void, Void)
		end

	make_from_host (a_host: EPX_HOST) is
			-- Prepare for request to resolved `a_host'. If `port' is 0,
			-- the default port is taken, else the port can be overruled.
		require
			host_found: a_host /= Void and then a_host.found
		do
			make_from_host_and_port (a_host, 0)
		end

	make_from_host_and_port (a_host: EPX_HOST; a_port: INTEGER) is
			-- Prepare for request to `a_host'. If `port' is 0, the
			-- default port is taken, else the port can be overruled.
		require
			host_found: a_host /= Void and then a_host.found
			valid_port: a_port >= 0 and a_port <= 65535
		local
			service: EPX_SERVICE
		do
			make_with_port (a_host.name, a_port)
			host := a_host
			create sa.make (host, service)
		end


feature -- Access

	default_port: INTEGER is 80
			-- Default port for this protocol.

	default_port_name: STRING is
			-- Default port name for this protocol.
		do
			Result := once_http
		end

	raw_response: STRING
			-- Raw response if `read_raw_response' has been called


feature -- Open and close

	open is
			-- Open connection to `server_name'. If authorisation, if
			-- supported by the protocol, fails, connection is closed.
			-- Check `last_reply_code' for details.
		local
			tcp: EPX_TCP_CLIENT_SOCKET
		do
			if sa = Void then
				precursor
			else
				create tcp.make
				tcp.set_continue_on_error
				tcp.open_by_address (sa)
				if tcp.is_open then
					tcp.set_raise_exception_on_error
					http := tcp
				end
			end
		end

	close is
		do
			is_authenticated := False
			precursor
		end


feature -- Requests

	get (path: STRING) is
			-- Send GET request to server. Use `read_response' to fetch
			-- the response.
		require
			path_not_empty: path /= Void and then not path.is_empty
		deferred
		end


feature -- Response

	read_response is
			-- Read entire resonse.
		require
			open: is_open
		deferred
		end

	read_raw_response is
			-- Read entire resonse, but don't parse or interpret it in
			-- any way. Put the response in `raw_response'.
		require
			open: is_open
		do
			create raw_response.make (8192)
			from
				http.read_string (8192)
			until
				http.eof
			loop
				raw_response.append_string (http.last_string)
				http.read_string (8192)
			end
		end


feature -- Client http version

	client_version: STRING is
			-- Client's version of the http protocol
		deferred
		ensure
			have_version: Result /= Void and then not Result.is_empty
		end


feature {NONE} -- Implementation

	authenticate is
		do
			is_authenticated := True
		end

	assert_closed is
			-- Make sure http is no longer connected.
		do
			if is_open then
				close
			end
		ensure
			http_closed: http = Void
		end

	assert_http_closed is
		obsolete	"20040-12-16 use assert_closed"
		do
			assert_closed
		end

	sa: EPX_HOST_PORT
			-- Binding of host to http port

	is_authenticated: BOOLEAN


invariant

	host_found: sa /= Void implies host /= Void and then host.found

end
